#!/usr/bin/perl

################################################################################
# create_lost_and_found.pl
#
# The purpose of this script is to cleanup the last user home folder (/Users/<name>).
#
# Copyright (c) 2002 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
#
################################################################################

################################################################################
# This script deals with the old home folder (ex. /Users/guest).
#          On kiosks, the home folder is deleted.  On other computers the old 
# home folder (ex /Users/guest) is moved to the "lost and found".  The owner of 
# the old home folder is found from the file ~/.username, which is created by 
# this script (search for $login_name_trigger below).  The lost and found folder 
# is named after the owner (ex. 
# /etc/entman/system/home_folder/lost_and_found/joe1234).
#          And, it does this really bad cleanup job on the folder that was 
# just moved to /etc/entman/system/home_folder/lost_and_found (this code needs to be 
# improved, and there is untested code already to do this, but we haven't had 
# time to test it).  
################################################################################

use lib "/Library/Xhooks/Modules/";
require "xhooks/lib/config.pl";
require "edu.utah.scl.home_folder/lib/home_folder_lib.pl";

use File::Find;

$template_homefolder_files = "/Library/Preferences/Xhooks/Caches/edu.utah.scl.home_folder/home_folder_files";
$homefolder_lostandfound = "Desktop/Lost & Found";

#$lostAndFoundDays = "14"; # Number of days items will be saved in a lost and found - NOT IMPLEMENTED YET!!!

################################################################################
# GO:
my $rightnow = time;
entman_logger ( "log", "Started..." );

my $print_debug = 1;
 
# Get stuff from entman_config.pl
if ( $this_computer_is_a_kiosk_computer || $this_computer_is_a_server ) {
  $createLostAndFound = 0;
} else {
  $createLostAndFound = 1;
}

if ( -e $userHomeFolder ) {  # $userHomeFolder ex. /Users/student

  if ( $assimilate_keep_lost_and_found_switch and $createLostAndFound ) {

    # Get the username of the last user
    if ( -e "$userHomeFolder/$login_name_trigger" ) {
      # Keep a lost and found
      $lastLoginTimeStamp = getTimeStamp( getFileModTime( "$userHomeFolder/$login_name_trigger" ) ); # /Users/student/.username
      $lastUsername = readOneLineFile( "$userHomeFolder/$login_name_trigger" ); # ex "abc1234" (typical university id... no?)

      # Create the hidden lost and found folder for this user
      if ( ! -e "$global_home_folder_lost_and_found/$lastUsername" ) { # /etc/entman/system/home_folder/lost_and_found/abc1234
        print "mkdir \"$global_home_folder_lost_and_found/$lastUsername\"\n" if $print_debug;
        system "mkdir \"$global_home_folder_lost_and_found/$lastUsername\"";
        print "/usr/sbin/chown -R $lastUsername \"$global_home_folder_lost_and_found/$lastUsername\"\n" if $print_debug;
        system "/usr/sbin/chown $lastUsername \"$global_home_folder_lost_and_found/$lastUsername\"";
      }

      # Move the old folder to the hidden lost and found
      print "/bin/mv \"$userHomeFolder\" \"$global_home_folder_lost_and_found/$lastUsername/$lastLoginTimeStamp\"\n" if $print_debug;
      system "/bin/mv \"$userHomeFolder\" \"$global_home_folder_lost_and_found/$lastUsername/$lastLoginTimeStamp\"";
      
      if ( ! -e "$global_home_folder_lost_and_found/$lastUsername/$lastLoginTimeStamp/.dontcleanup" ) {
        # Cleanup old home folder
        cleanup_old_home_folder ( "$global_home_folder_lost_and_found/$lastUsername", $lastLoginTimeStamp );
      }

    } else {

      # This will happen if the system makes a home folder.  Which happens ironically every now and then.
      entman_logger ( "log", "Alert, $userHomeFolder/$login_name_trigger missing.  Deleting home folder." );
      
      # Delete folder
      put_in_trash( $userHomeFolder );

    }

  } else {

    # Don't keep a lost and found
    put_in_trash( $userHomeFolder );

  }
}

$totaltime = time - $rightnow;
entman_logger ( "log", "Done.  Took $totaltime." );

exit 0;

sub delete_folder {
  my ( $my_folder ) = @_;

  print "chflags -R nouchg \"$my_folder\"\n" if $print_debug;
  system "chflags -R nouchg \"$my_folder\"";
  print "rm -rf \"$my_folder\"\n" if $print_debug;
  system "rm -rf \"$my_folder\"";
  
}

####################################
# STEP 4 - DEAL WITH LOST AND FOUND
# If this doesn't work, it is probably because you are using special characters 
# (other than " " or "&", which this script will fix).

#
#  THIS SUBROUTINE NEEDS SERIOUS FIXING UP
#

sub cleanup_old_home_folder {
  my ( $user_lost_and_found, $my_folder ) = @_;
  my $path_to_home_folder = "$user_lost_and_found/$my_folder";

  ####################################
  # Move any old timestamped Lost and Found folders out of Desktop/Lost & Found up a level
  print "do i exist $path_to_home_folder/$homefolder_lostandfound\n" if $print_debug;
  if ( -e "$path_to_home_folder/$homefolder_lostandfound" ) { # "~/Desktop/Lost & Found";
    # mv ~/Desktop/Lost\ \&\ Found/* ~/..;
    @dirs = getFileFolderList( "$path_to_home_folder/$homefolder_lostandfound/" );
    foreach $i ( @dirs ) {
      print "/bin/mv \"$path_to_home_folder/$homefolder_lostandfound/$i\" \"$user_lost_and_found/\"\n" if $print_debug;
      system "/bin/mv \"$path_to_home_folder/$homefolder_lostandfound/$i\" \"$user_lost_and_found/\"";
    }
    system "/bin/rm -rf \"$path_to_home_folder/$homefolder_lostandfound\""; # rm -rf ~/Desktop/Lost\ \&\ Found
  }

  ####################################
  # CLEAN UP THE OLD HOME FOLDER

  # Delete Known throw away folders
  #if ( not test box!! ) {
    @home_folder_auto_delete_these_folders = ("Library", "Shared/Library", "SorensonMedia/Squeeze", );
    foreach $auto_detele ( @home_folder_auto_delete_these_folders ) {
      put_in_trash( "$path_to_home_folder/$auto_detele" );
    }
  #}

  ####################################
  # Everything else

  chdir $path_to_home_folder;

  if ( open FILE, "< $template_homefolder_files" ) {
    while ( <FILE> ) {
      my $line = $_;
      my @parts = split / /, $line;
      my $path = unescape_radmind_path ( $parts[0] );
      my $test = 1;
      my $index;
      foreach $auto_detele ( @home_folder_auto_delete_these_folders ) {
        $index = index $path, $auto_detele;
        if ( $index > 0 and $index < 3 ) { $test = 0; last; }
      }
      if ( $test and -e $path ) {
        if ( ! unlink $path ) {
          system "/usr/bin/chflags -R nouchg \"$path\"";
          unlink $path or entman_logger ("log", "Couldn't unlink $path\n");# if $print_debug;
        }
      }
    }
    close FILE;
  } else {
    entman_logger ("log", "Can't open $template_homefolder_files: $!");
    return;
  }

  # Remove Finder created files (.DS_Store, .localized) # -type l -or 
  system "/usr/bin/find ./ \\( -name \".DS_Store\" -or -name \".localized\" \\) -delete";

  # Remove empty directories
  finddepth(sub{rmdir},'.');

  # cleanup Trash, make visible if stuff is in it
  if ( -e "$path_to_home_folder/.Trash" ) {
    system "/bin/mv \"$path_to_home_folder/.Trash\" \"$path_to_home_folder/Trash\"";
    system "$entman_folder/bin/SetFile", "-a", "v", "$path_to_home_folder/Trash";
  }

  # cleanup root folder
  system "/bin/rm -f \"$path_to_home_folder/$login_name_trigger\"";
  if ( findNumberOfFoldersAndFiles( "$path_to_home_folder" ) < 1 ) {
    system "/bin/rm -rf \"$path_to_home_folder\"";
  }

  # If there is nothing in the lost and found, well, just delete the whole thing.
  if ( findNumberOfFoldersAndFiles( "$user_lost_and_found" ) < 1 ) {
    system "/bin/rm -rf \"$user_lost_and_found\"";
  }

}

  ####################################
  # DO GENERAL LOST AND FOUND CLEANUP

  # remove folders that are too old
  #@dirs = getFileFolderList( "/Users" );
  #foreach $i ( @dirs ) {
  #  if ( -M "$global_home_folder_lost_and_found/$i" > $lostAndFoundDays ) {
  #    system "/bin/rm -rf \"$global_home_folder_lost_and_found/$i\"";
  #  }
  #}

  # remove folders that are too big
  #@dirs = getFileFolderList( "/Users" );
  #foreach $i ( @dirs ) {
  #  if ( -M "$global_home_folder_lost_and_found/$i" > $lostAndFoundDays ) {
  #    #print "/bin/rm -rf \"$global_home_folder_lost_and_found/$i\"";
  #  }
  #}
