#!/usr/bin/perl

################################################################################
# newPresentation.pl
#
# This script is for a kiosk like Mac that runs powerpoint presentations.
#
# This script checks for PowerPoint files in a dropbox and moves the old
# powerpoint file to an old folder and moves the new file to the current folder
# and then launches the new file.  
#
# Copyright (c) 2002 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
#
################################################################################

use lib "/etc/entman/conf/";
require "entman_config.pl";

################################################################################
# Change these
#
my $debug = 0;

# note these paths cant have spaces!!!
my $pathToCurrentPresentations = "/Volumes/Shared/current";
my $pathToNewPresentations = "/Volumes/Shared/new";
my $pathToOldPresentations = "/Volumes/Shared/old";
my $good_fileExtension = ".pps";
my $good_fileType = "PPSS";
my $good_fileCreator = "PPT3";

my $path_to_powerpoint = "/Applications/Microsoft Office X/Microsoft PowerPoint";
my $valid_update = 0;
my ($fn, $ext, $dir);

################################################################################
# Go
#
$username = getUsername();
if ( $username ne $kioskusername or ! $this_computer_is_a_presentation_computer or isThisAppRunning( "run_radmind", "root" ) ) {
	exit;
}

# Make sure all the folders are there.
if ( ! -e $pathToCurrentPresentations ) {
	system "/bin/mkdir $pathToCurrentPresentations";
}
if ( ! -e $pathToNewPresentations ) {
	system "/bin/mkdir $pathToNewPresentations";
}
if ( ! -e $pathToOldPresentations ) {
	system "/bin/mkdir $pathToOldPresentations";
}
# Fix permissions just in case.
system "/bin/chmod -R 755 \"$pathToCurrentPresentations\"";
system "/bin/chmod -R 755 \"$pathToOldPresentations\"";
system "/bin/chmod -R 733 \"$pathToNewPresentations\"";

# Find out if there is an update  
opendir(NEW, $pathToNewPresentations) or die "Error opening $pathToNewPresentations: $!";
my @list = grep {$_ ne '.' and $_ ne '..' and $_ ne '.DS_Store'} readdir NEW;
my $countOfFiles = $#list + 1;
closedir NEW;

if ( $countOfFiles == 1 ) {
	my $file = $list[0];

	($fn, $dir, $ext) = fileparse($file, qr/\.[^.].*/);

	my $creator = `\usr/local/bin/GetFileInfo -c \"$pathToNewPresentations/$file\"`;
	my $type = `/usr/local/bin/GetFileInfo -t \"$pathToNewPresentations/$file\"`;
	$creator = substr ( $fileCreator, 1, 4 );
	$type = substr ( $fileType, 1, 4 );

	if ( $ext eq $good_fileExtension ) {
		$valid_update = 1;
	} elsif ( $creator eq $good_fileCreator and $type eq $good_fileType ) {
		$valid_update = 1;
	}
	
	if ( $valid_update ) {

		################
		# UPDATE PRESENT
		################

		say( "There is a new presentation file" );

		my $timeStamp = getTimeStamp( time );

		system "/bin/mkdir \"$pathToOldPresentations/$timeStamp\"" or die "Could not create $pathToOldPresentations/$timeStamp: $!";

		system "/bin/mv $pathToCurrentPresentations/* \"$pathToOldPresentations/$timeStamp/\" 2&>/dev/null";
		system "/bin/mv $pathToNewPresentations/* $pathToCurrentPresentations/" or die "Could not move $pathToNewPresentations/* $pathToCurrentPresentations: $!";
		openPresentation();

	} else {

		#################
		# WRONG FILE TYPE
		#################

		say( "There is a new presentation file, but it is the wrong file type" );

		my $timeStamp = getTimeStamp( time );
		system "/bin/mkdir \"$pathToOldPresentations/$timeStamp\"" or die "Could not create $pathToOldPresentations/$timeStamp: $!";
		system "/bin/mv $pathToNewPresentations/* \"$pathToOldPresentations/$timeStamp/\"";
		system "/bin/echo \"Wrong file type\" > \"$pathToOldPresentations/$timeStamp/ README\"";
		
	}

} elsif ( $countOfFiles > 1 ) {

	#################
	# TO MANY UPDATES
	#################

	say( "There is a new presentation file, but there are too many files" );

	$timeStamp = getTimeStamp( time );
	system "/bin/mkdir \"$pathToOldPresentations/$timeStamp\"";
	system "/bin/mv $pathToNewPresentations/* \"$pathToOldPresentations/$timeStamp/\"";
	system "/bin/echo \"Too many files in the new folder\" > \"$pathToOldPresentations/$timeStamp/ README\"";

} else {
	if ( ! isThisAppRunning( "PowerPoint", $username ) ) {
		system "/bin/chmod -R 755 $pathToCurrentPresentations";
		system "open '$path_to_powerpoint'";
		openPresentation();
		#system "/usr/bin/open $pathToCurrentPresentations/*";
	} else {
		$frontMost = `osascript -e 'tell application "System Events" to set x to application processes whose frontmost is true'`;
		chomp $frontMost;
		if ( $frontMost ne "application process Microsoft PowerPoint" ) {
		  openPresentation();
		}
	}
}

say( "newPresentation done" );
exit 0;


################################################################################
sub openPresentation {
	# kill PowerPoint (because MS did not make PowerPoint scriptable).
	@pid_list = getPID( "PowerPoint", $username );
	$_ = substr ( $pid_list[0], 0, 6 );
	system "/bin/kill $_";
	#
	# open new presentation
	system "/bin/chmod -R 755 $pathToCurrentPresentations";
	system "/usr/bin/open $pathToCurrentPresentations/*";
}
