#!/usr/bin/perl

################################################################################
# Copyright (c) 2008 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

# Writes /Library/Preferences/com.apple.loginwindow.plist and makes sure that
# the machine will autologin if it is suppose to (if the autologin user logs out
# and the machine is restarted it will NOT autologin without this script...)

use lib "/Library/Xhooks/Modules/";
require "xhooks/lib/config.pl";

use Foundation;
require "xhooks/lib/perlplist.pl";

my $disable_console_access = ( defined $LOGINWINDOW_DISABLE_CONSOLE_ACCESS and $LOGINWINDOW_DISABLE_CONSOLE_ACCESS eq "True" ) ? 1:0;
my $disable_poweroff = ( defined $LOGINWINDOW_DISABLE_POWEROFF and $LOGINWINDOW_DISABLE_POWEROFF eq "True" ) ? 1:0;

write_autologin_file();

entman_logger ( "log drawer", "Wrote com.apple.loginwindow.plist" );

exit 0;

sub write_autologin_file {

  my $preffile = "/Library/Preferences/com.apple.loginwindow.plist";

  # Load pref
  if ( -e $preffile) {
    $dict = loadDefaults( $preffile );
    setPlistObject( $dict, "DisableConsoleAccess", cocoaBool($disable_console_access?1:0) );
    setPlistObject( $dict, "PowerOffDisabled", cocoaBool($disable_poweroff?1:0) );
    setPlistObject( $dict, "RetriesUntilHint", cocoaInt(0) );
    setPlistObject( $dict, "SHOWFULLNAME", cocoaBool(1) );
    setPlistObject( $dict, "noPasswordKey", cocoaBool(0) );

  } else {
    my %pref = (
                 "DisableConsoleAccess"	=>	cocoaBool($disable_console_access?1:0),
                 "PowerOffDisabled"		=>	cocoaBool($disable_poweroff?1:0),
                 "RetriesUntilHint"		=>	cocoaInt(0),
                 "SHOWFULLNAME"			=>	cocoaBool(1),
                 "lastUser"				=>	"loggedOut",
                 "lastUserName"			=>	$adminusername,
                 "noPasswordKey"		=>	cocoaBool(0),
               );
    $dict = cocoaDictFromPerlHash( %pref );
  }

  # Autologin
  if ( $this_computer_is_authenticated ) {
    removePlistObject ( $dict, "autoLoginUser" );
    removePlistObject ( $dict, "autoLoginUserUID" );
  } else {
    removePlistObject ( $dict, "autoLoginUserUID" );
    setPlistObject( $dict, "autoLoginUser", $kioskusername );
    setPlistObject( $dict, "lastUser", "Restart" );
  }

  # AdminHostInfo.  Valid values are:
  # HostName, SystemVersion, SystemBuild, SerialNumber, IPAddress, DSStatus, or Time
  if ( -e "$attributes_folder/loginwindow_adminhostinfo" ) {
    $adminhostinfo = `cat \"$attributes_folder/loginwindow_adminhostinfo\"`;
    chomp $adminhostinfo;
    if ( $adminhostinfo ne "" ) {
      setPlistObject( $dict, "AdminHostInfo", $adminhostinfo );
    }
  }

  # NetworkAuthConfigList (802.1x authentication at the loginwindow)
  # Valid values are NDA as of 2005.12.01... sorry.
  if ( -e "$attributes_folder/802.1x_loginwindow" ) {
    my $newdict = loadDefaults( "$attributes_folder/802.1x_loginwindow" );
    my $NetworkAuthConfigList = getPlistObject( $newdict, "NetworkAuthConfigList" );
    setPlistObject( $dict, "NetworkAuthConfigList", $NetworkAuthConfigList );
  }

  #printObject ($dict);

  if ( ! saveDefaults( $dict, $preffile ) ) {
    entman_logger ( "log", "Could not write to file $preffile" );
  }

  system "/usr/bin/plutil -convert binary1 \"$preffile\"";

}
