#!/usr/bin/perl

################################################################################
# Copyright (c) 2008 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
################################################################################

# hack_loginpanel.pl
#
# This script changes the loginwindow to show the status of maintenance and a
# customized graphic (displaying the OS version or a dated graphic for holidays etc).
#
# This script is executed by status/SED_find_errors.pl
#
# The first param is text that will be displayed on the loginpanel.
# If a second param is present, it will use the "error" footer graphic.
#
# You must profide the graphics!!! (I doubt you want to use the U of U's).
# The graphics and other files go in /Library/Preferences/Xhooks/edu.utah.scl.loginpanel
#
# You must also copy modified Loginpanel NIBS...



# our template nib has been `plutil -convert`ed to XML.  We patch it with the 
# desired value, convert it back to binary, then move it into place.

################################################################################

# GET OS VERSION (this must be done first)
my $osnum = `sw_vers | grep ProductVersion | awk '{print \$2}'`;
chomp $osnum;
my $osnum_short = $osnum;
$osnum_short =~ s/(\d+\.\d+).*/$1/;

die "Couldn't get OS version!" if $osnum eq "" or $osnum_short eq "";

my $core_services = "/System/Library/CoreServices";
my $basepath = "/Library/Preferences/Xhooks/edu.utah.scl.loginpanel/${osnum_short}";

################################################################################

# CHANGE THE FOLLOWING IF YOU DON'T USE THE NAMES PROVIDED
#
# THESE ARE ALL FILES THAT YOU NEED TO CREATE!
#
my $loginpanel_footer_location_good = "$basepath/loginpanel_footer_good.tif";
my $loginpanel_footer_location_bad = "$basepath/loginpanel_footer_error.tif";
my $loginpanel_footer_autologin_location_good = "$basepath/loginpanel_footer_good_autologin.tif";
my $loginpanel_footer_autologin_location_bad = "$basepath/loginpanel_footer_error_autologin.tif";
my $loginpanel_footer_server_location_good = "$basepath/loginpanel_footer_good_server.tif";
my $loginpanel_footer_server_location_bad = "$basepath/loginpanel_footer_error_server.tif";
my $loginwindow_header = "$basepath/loginpanel_header_${osnum}.tif";
if ( ! -e $loginwindow_header ) {
  $loginwindow_header = "$basepath/loginpanel_header_${osnum_short}.tif";
}
my $login_nib_template = "$basepath/LoginWindowUI.nib/keyedobjects.nib";

# This is optional:
my $dated_material_folder = "$basepath/dated_material/";

################################################################################
# No need to change anything else

use lib "/Library/Xhooks/Modules/";
require "xhooks/lib/config.pl";

my $login_nib_new = "$core_services/SecurityAgentPlugins/loginwindow.bundle/Contents/Resources/English.lproj/LoginWindowUI.nib/keyedobjects.nib";
my $login_nib_username_new = "$core_services/SecurityAgentPlugins/loginwindow.bundle/Contents/Resources/English.lproj/NameAndPassword.nib/keyedobjects.nib";

my $loginpanel_message = "/Library/Preferences/Xhooks/triggerfiles/loginpanel_message";

my $loginpanel_footer_location = "$core_services/SecurityAgent.app/Contents/Resources/MacOSX.tif";
#10.5:
#/System/Library/CoreServices//SecurityAgent.app/Contents/Resources/MacOSX.tif # there is no file at this location by default
#./SecurityAgentPlugins/loginwindow.bundle/Contents/Resources/MacOSX.tif # this is the default

# 10.4
my $loginpanel_header_location = "$core_services/SecurityAgent.app/Contents/Resources/applelogo.tif";
# 10.5
$loginpanel_header_location = "$core_services/SecurityAgentPlugins/loginwindow.bundle/Contents/Resources/applelogo.tif" if $osnum_short eq "10.5";

my $login_nib_replace_text="xxxxxxxx";

my $loginTitle = "        ";
my $error = 0;
if ( -e $loginpanel_message ) {
  if ( open FILE, "<$loginpanel_message" ) {
    my $line = <FILE>;
    close FILE;
    if ( $line =~ /(.*) (\d)/ ) {
      $loginTitle = sprintf ("%-8s", $1);
      $error = $2;
    }
  } else {
     print "ERROR: Could not read $loginpanel_message.";
  }
}

################################################################################
# Replace login panel graphic

# Use dated login panels if present:
my @diritems = get_rand_items_in_dated_folder_for_path( $dated_material_folder, 1 );
if ( defined $diritems[0] and -e $diritems[0] ) {
  $loginwindow_header = $diritems[0];
}

# Backup old stuff
if ( ! -e "${loginpanel_header_location}.bkup" ) {
  system "/bin/mv \"$loginpanel_header_location\" \"${loginpanel_header_location}.bkup\"";
}

#
if ( -e "$loginwindow_header" ) {

  system "/bin/cp \"$loginwindow_header\" \"$loginpanel_header_location\"";

  system "/bin/chmod 644 \"$loginpanel_header_location\"";
  system "/usr/sbin/chown 0:0 \"$loginpanel_header_location\"";

} else {
  entman_logger ( "log", "file $loginwindow_header missing." );
}

################################################################################
# Replace the footer to use

if ( defined $AUTOLOGIN and $AUTOLOGIN eq "True" ) {
  $loginpanel_footer_location_bad = $loginpanel_footer_autologin_location_bad;
  $loginpanel_footer_location_good = $loginpanel_footer_autologin_location_good;
} elsif ( defined $SERVER and $SERVER eq "True" ) {
  $loginpanel_footer_location_bad = $loginpanel_footer_server_location_bad;
  $loginpanel_footer_location_good = $loginpanel_footer_server_location_good;
}

if ( $error ) {
  $loginwindow_footer = $loginpanel_footer_location_bad;
} else {
  $loginwindow_footer = $loginpanel_footer_location_good;
}

if ( ! -e "${loginpanel_footer_location}.bkup" and -e $loginpanel_footer_location ) {
  system "/bin/mv \"$loginpanel_footer_location\" \"${loginpanel_footer_location}.bkup\"";
  #print "/bin/mv \"$loginpanel_footer_location\" \"${loginpanel_footer_location}.bkup\"\n";
}

if ( -e "$loginwindow_footer" ) {

  system "/bin/cp \"$loginwindow_footer\" \"$loginpanel_footer_location\"";
  #print "/bin/cp \"$loginwindow_footer\" \"$loginpanel_footer_location\"\n";

  system "/bin/chmod 644 \"$loginpanel_footer_location\"";
  system "/usr/sbin/chown 0:0 \"$loginpanel_footer_location\"";

} else {
  entman_logger ( "log", "file $loginwindow_footer missing." );
}

################################################################################
# Change the text in the loginwindow

# don't do anything if the search and replace text are different sizes

if ( length( $loginTitle ) == length( $login_nib_replace_text ) ) {

  # read the template file
  open ( INFILE, "<$login_nib_template" );
  binmode ( INFILE );
  $i=0;
  while ( read ( INFILE,  $buffer[$i], 1 ) ) {
    $i++;
  }
  close ( INFILE );

  # replace the text
  $login_nib_replace_text_length = length ( $login_nib_replace_text );
  for ( $i = 0; $i <= $#buffer; $i++ ) {
    @buffer2 = @buffer[$i .. $i+$login_nib_replace_text_length-1];
    if ( join ( "",@buffer2 ) eq $login_nib_replace_text ) {
      splice( @buffer, $i, $login_nib_replace_text_length, $loginTitle );
      $i = $#buffer + 1;
    }
  }

  # write the file
  unlink "/tmp/loginwindow.nib" if -e "/tmp/loginwindow.nib";
  open ( OUTFILE, ">/tmp/loginwindow.nib" );
  binmode ( OUTFILE );
  for ( $i = 0; $i < $#buffer; $i++ ) {
    print OUTFILE $buffer[$i];
  }
  close ( OUTFILE );

  # Check if there was an error, or if there is even a difference
  if ( -e "/tmp/loginwindow.nib" ) {

    if ( `/usr/bin/diff \"/tmp/loginwindow.nib\" \"$login_nib_username_new\"` ) {

      if ( ! -e "${login_nib_new}.bkup" ) {
        system "/bin/mv \"$login_nib_new\" \"${login_nib_new}.bkup\"";
        #print "/bin/mv \"$login_nib_new\" \"${login_nib_new}.bkup\"\n";
      }

      system "/bin/mv \"/tmp/loginwindow.nib\" \"$login_nib_new\"";
      #print "/bin/mv \"/tmp/loginwindow.nib\" \"$login_nib_new\"\n";

      system "/bin/chmod 644 \"$login_nib_new\"";
      system "/usr/sbin/chown 0:0 \"$login_nib_new\"";

    } else {
      unlink "/tmp/loginwindow.nib";
      #print "/tmp/loginwindow.nib is the same as $login_nib_new\n";
    }
  } else {
    entman_logger ( "log", "/tmp/loginwindow.nib did not get created for an unknown reason." );
  }

} else {
  entman_logger ( "log", "The title $loginTitle is different than what I'm suppose to replace $login_nib_replace_text." );
}

entman_logger ( "log", "Done." );

exit 0;
