#################################################################################
# common.sh
#
# Copyright (c) 2002 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
#
################################################################################

entman_logger () {
    case "$1" in
    drawer|stderr)
        shift; echo "$*" 1>&2 ;;
    log)
        shift; logger -is -t \"entman-`basename $0`\" -p user.warn "$*" 1>&2 ;;
    window|stdout)
        shift; echo "$*" ;;
    *)
        logger -s -t \"entman-`basename $0`\" -p user.warn "$*" 1>&2 ;;
    esac
}

runhook () {
	local hook="$1"
	local detach="$2"
	shift 2
	
	if [ -s ${hook} -a -x ${hook} ]; then
		my_logger "log drawer" "Executing ${hook}..."

		if [ "$detach" = "true" ]; then

			${hook} "$@" &

		else

			# run the item
			${hook} "$@"

			if [ $? -ne 0 ]; then
				my_logger "log drawer" "${hook} failed!"
			fi
		fi

	else 
		my_logger "log drawer" "${hook} could not be executed!"
	
	fi
}

launch_runhook_script () {
	local use_ihook="$1"
	local ihook_is_lazy="$2"
	local hook_script_output="$3"
	local hook_pid_file="$4"

	ihook_window_title=""
	ihook_window_size=""

	ihook_lazy_time=10
	lazy_ihook="/Library/Xhooks/Modules/xhooks/bin/lazy_ihook.sh"
	hook_pid_check="/Library/Xhooks/Modules/xhooks/bin/pidcheck.sh"
	hook_script="/Library/Xhooks/Modules/xhooks/bin/runhooks.sh"
	
	if [ "${use_ihook}" = 1 -a -x ${ihook} ]; then
	
		if [ "${ihook_is_lazy}" = 1 ]; then
		
			my_logger "stderr" "Using lazy iHook"
		
			# Make sure we are the owner of the script output file
			[ -e "${hook_script_output}" ] && rm "${hook_script_output}"
			touch "${hook_script_output}"
	
			# run the lazy ihook script
			(sleep ${ihook_lazy_time}; ${lazy_ihook} -f "${hook_pid_check} ${hook_pid_file} ${hook_script}" "${hook_script_output}" "${ihook_window_title}" "${ihook_window_size}") &
			spawnpid=$!
	
			# run the script
			( ${hook_script} -pidfile "${hook_pid_file}" "$@" 3>&1 1>&2 2>&3 | perl -ple 'BEGIN{$|=1;}s/^/ERROR: /' ) >> ${hook_script_output} 2>&1
	
			# kill the iHook script
			kill ${spawnpid}
	
		else
	
			my_logger "stderr" "Using iHook"
	
			# run iHook without a titlebar and pass all parameters to the script   
			${ihook} --no-titlebar --script=${hook_script} "$@"
	
		fi
	
	else
	
		my_logger "stderr" "Not using iHook"
	
		${hook_script} "$@"
	
	fi
}
