#################################################################################
# config.sh
#
# Xhooks admin configuration for shell scripts
#
# Copyright (c) 2002 University of Utah Student Computing Labs.
# All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appears in all copies and
# that both that copyright notice and this permission notice appear
# in supporting documentation, and that the name of The University
# of Utah not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission. This software is supplied as is without expressed or
# implied warranties of any kind.
#
################################################################################

################################################################################
# This file controls what Entman "hooks" and modifications run.  In a nutshell,
# the hooks are:

#  /Library/Xhooks/Modules/xhooks/bin/startupearly.hook    (the startup item)
#    (hooked by /etc/rc.login on 10.4, by /System/Library/LaunchDaemons/com.apple.loginwindow.plist on 10.5)
#  /Library/Xhooks/Modules/xhooks/bin/login.hook    (the loginhook)
#  /Library/Xhooks/Modules/xhooks/bin/logout.hook   (the logouthook)
#    (both hooked by /var/root/Library/Preferences/com.apple.loginwindow.plist)
#
#  /Library/LaunchDaemons/edu.utah.scl.xhooks.*
#    By default this includes an idle script and a startup late script.
#
# If some other installer modifies these files, you may lose the entman
# changes.
# 
# NOTE: Many many many other system modifications occur when you hook the startup
# item, loginhook, and logouthook, and naturally when you run post_maintenance
# etc...  So DON'T RUN THIS ON A PRODUCTION COMPUTER until you have tested and are
# confident you wont get someone flamin mad.  This whole package is designed to 
# give the admin great powers, so use it wisely.
#
# This file also contains some config info for launch_maintenance_script,
# which runs radmind, and the report_to_master.sh script, which sends messages
# to a webdatabase using curl.
#
##

# If you move entman, change this to its path
# Also change the same path that is in these files:
#   /etc/entman/conf/radmind/create_overload_filter
#   /etc/entman/conf/entman_config.pl
#   /etc/entman/conf/entman_global_config.pl
#   /Library/Preferences/com.apple.loginwindow.plist (if you don't run post_maintenance.pl)

ENTMAN_FOLDER="/etc/entman"
XHOOKS_FOLDER="/Library/Xhooks"

# If you want to disable all of the Entman stuff, change the following to 0
# Note, the startup item, loginhook, logout hook will still run.
# This also does not disable the crontab file (so if there is stuff in there you
# will have to manually change it).  Nor does it enable the ">console" user
# login (which is disabled by Entman).  This is just a soft switch that I tried to get
# all of the scripts to honor.

ENTMAN_ENABLED=1

#################################################################################
# Entman Hooks (these are the files called by the system)
##

# STARTUP
ENTMAN_STARTUP_IHOOK=0			# use iHook?
ENTMAN_STARTUP_IHOOK_LAZY=1		# if using iHook, always show (0) or only show if takes >10 seconds
ENTMAN_STARTUP_OUTPUT="/var/log/startup_early.out"
ENTMAN_STARTUP_TRIGGERFILE="/Library/Preferences/Xhooks/triggerfiles/startupscript_done_running"
ENTMAN_STARTUP_PID_FILE="/Library/Preferences/Xhooks/triggerfiles/startup_pid"
ENTMAN_STARTUP_IHOOKBACKGROUND=""
ENTMAN_STARTUP_IHOOK_TITLE=""
ENTMAN_STARTUP_IHOOK_SIZE=""
ENTMAN_STARTUP_BATON="/Library/Preferences/Xhooks/triggerfiles/startup_early_late_baton";

# LOGIN
ENTMAN_LOGINHOOK_IHOOK=0		# use iHook?
ENTMAN_LOGINHOOK_IHOOK_LAZY=1	# if using iHook, always show (0) or only show if takes >10 seconds
ENTMAN_LOGINHOOK_OUTPUT="/var/log/loginhook.out"
ENTMAN_LOGINHOOK_TRIGGERFILE="/Library/Preferences/Xhooks/triggerfiles/loginhook_done_running"
ENTMAN_LOGINHOOK_PID_FILE="/Library/Preferences/Xhooks/triggerfiles/login_pid"
ENTMAN_LOGINHOOK_IHOOKBACKGROUND=""
ENTMAN_LOGINHOOK_IHOOK_TITLE=""
ENTMAN_LOGINHOOK_IHOOK_SIZE=""

ENTMAN_LOGINHOOK_RUNNING="/Library/Preferences/Xhooks/triggerfiles/login_hook_running"

# LOGOUT
ENTMAN_LOGOUTHOOK_IHOOK=0		# use iHook?
ENTMAN_LOGOUTHOOK_IHOOK_LAZY=1	# if using iHook, always show (0) or only show if takes >10 seconds
ENTMAN_LOGOUTHOOK_OUTPUT="/var/log/logouthook.out"
ENTMAN_LOGOUTHOOK_TRIGGERFILE="/Library/Preferences/Xhooks/triggerfiles/logouthook_done_running"
ENTMAN_LOGOUTHOOK_PID_FILE="/Library/Preferences/Xhooks/triggerfiles/logout_pid"
ENTMAN_LOGOUTHOOK_IHOOKBACKGROUND=""
ENTMAN_LOGOUTHOOK_IHOOK_TITLE=""
ENTMAN_LOGOUTHOOK_IHOOK_SIZE=""

ENTMAN_LOGOUTHOOK_RUNNING="/Library/Preferences/Xhooks/triggerfiles/logout_hook_running"
ENTMAN_LOGOUTHOOK_FINISHED="/Library/Preferences/Xhooks/triggerfiles/logout_hook_finished"
ENTMAN_LOGOUTHOOK_PARENTSCRIPT="$XHOOKS_FOLDER/Modules/xhooks/bin/logout.hook"

# POSTMAINTENANCE
ENTMAN_POSTM_IHOOK=1		# use iHook?
ENTMAN_POSTM_IHOOK_LAZY=0	# if using iHook, always show (0) or only show if takes >10 seconds
ENTMAN_POSTM_OUTPUT="/var/log/postmaintenance.out"
ENTMAN_POSTM_TRIGGERFILE="/Library/Preferences/Xhooks/triggerfiles/postm_done_running"
ENTMAN_POSTM_PID_FILE="/Library/Preferences/Xhooks/triggerfiles/postm_pid"
ENTMAN_POSTM_IHOOKBACKGROUND="/Library/Preferences/Xhooks/configure/performing_maintenance1.gif"
ENTMAN_POSTM_IHOOK_TITLE="Post Radmind Maintenance"
ENTMAN_POSTM_IHOOK_SIZE=""


# OTHER
ENTMAN_IHOOK_PATH="/usr/local/bin/iHook.app/Contents/MacOS/iHook"
ENTMAN_LAZY_IHOOK="$XHOOKS_FOLDER/Modules/xhooks/bin/lazy_ihook.sh"
ENTMAN_LAZY_IHOOK_DELAY_TIME=10 # seconds
ENTMAN_HOOK_PID_CHECK="$XHOOKS_FOLDER/Modules/xhooks/bin/pidcheck.sh"







# remove when all blabla2.hook scripts are integrated into runhooks.sh
ENTMAN_HOOKSDIR="${XHOOKS_FOLDER}"


ENTMAN_TECH_TRIGGER="/Library/Preferences/Xhooks/triggerfiles/tech"

RUN_STARTUP_POST_MAINTENANCE="/Library/Preferences/Xhooks/triggerfiles/run_startup_post_maintenance"

# delete these files at startup
STARTUP_DELETE_FILES=(
/Library/Preferences/Xhooks/triggerfiles/creatingHomeFolders
/Library/Preferences/Xhooks/triggerfiles/movingHomeFolders
/tmp/reboot_required
)

POSTM_DELETE_FILES_FIRST=(
/Library/Preferences/Xhooks/triggerfiles/run_maintenance
)

POSTM_DELETE_FILES_LAST=(
$RUN_STARTUP_POST_MAINTENANCE
/var/log/edu.utah.scl.logoutsaver.stop
)

#################################################################################
##

ENTMAN_NETWORK_IS_UP="/Library/Preferences/Xhooks/triggerfiles/networkisup"

#################################################################################
# System Modifications:
##

CLEAN_VOLUMES_FOLDER=1 # This is used by /etc/rc.cleanup and if enabled, replaces
                       # /Volumes with a new folder at startup.
BYPASS_CDIS_STUFF=1    # This is used by /etc/rc to bypass importing the file
                       # /var/log/CDIS.custom (we don't tripwire this file).

#################################################################################
# Radmind stuff for launch_maintenance_script
##

RUN_RADMIND="/Library/Xhooks/Modules/edu.utah.scl.radmind/run_radmind.pl"
RADMIND_ULIMIT_VALUE=1024

#################################################################################
##
