/*
Copyright (c) 2002 Regents of The University of Michigan.
All Rights Reserved.

Permission to use, copy, modify, and distribute this software and
its documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appears in all copies and
that both that copyright notice and this permission notice appear
in supporting documentation, and that the name of The University
of Michigan not be used in advertising or publicity pertaining to
distribution of the software without specific, written prior
permission. This software is supplied as is without expressed or
implied warranties of any kind.

Research Systems Unix Group
The University of Michigan
c/o Wesley Craig
4251 Plymouth Road B1F2, #2600
Ann Arbor, MI 48105-2785

based loosely on the iHook source code
http://www.ihook.org
*/

#import "SPController.h"
#define SLIDETIME 7.0

static NSTimer	*timer = nil;
static int	curimg;

@implementation SPController

- ( id )init
{
    self = [super init];
    [NSApp setDelegate: self];
    pics = [[NSMutableArray alloc] init];
    return( self );
}

- ( void )awakeFromNib
{
    NSRect		contentRect = [mainView bounds];
	NSArray		*screens = [NSScreen screens];
	NSEnumerator *e = [screens objectEnumerator];
	NSScreen *screen;
	BOOL mainViewSet = NO;
	windows = [[NSMutableArray alloc] initWithCapacity:1];
	while( screen = [e nextObject] ) {
		SPWindow *myWindow = [[SPWindow alloc] initWithContentRect: contentRect
                                    styleMask: NSBorderlessWindowMask
                                    backing: NSBackingStoreBuffered
                                    defer: NO];
		if ( ! mainViewSet ) {
			[myWindow setContentView: mainView];
			mainViewSet = YES;
		} else {
		    SPView *myView = [[SPView alloc] initWithFrame:[screen frame]];
			[myWindow setContentView: myView];
		}
		[myWindow setLevel: ( NSScreenSaverWindowLevel - 1 )];
		[myWindow setHasShadow: YES];
		[myWindow setFrame: [screen frame] display: NO];
		[windows addObject:myWindow];
	}

	[NSCursor hide];
}

- ( void )applicationDidFinishLaunching: ( NSNotification * )aNotification
{
	NSEnumerator *e = [windows objectEnumerator];
	SPWindow *myWindow;
	while( myWindow = [e nextObject] ) {
		[myWindow makeFirstResponder: mainView];
		if ( [myWindow acceptsMouseMovedEvents] ) {
			NSLog( @"Accepts mouse moved" );
		}
		[myWindow makeKeyAndOrderFront: nil];
	}
    [self beginSlideShow];
}

- ( void )beginSlideShow
{
    int			i;
    NSArray		*picArray;
    NSImage		*bg;
    BOOL		isDir;
    NSFileManager	*manager;
    NSString		*picturesDir = [[NSBundle mainBundle]
                                        pathForResource: @"Pictures"
                                        ofType: nil];
                                        
    if ( picturesDir == nil ) {
        NSLog( @"Couldn't get pictures directory path!" );
        exit( 2 );
    }
    
    manager = [NSFileManager defaultManager];
    picArray = [manager directoryContentsAtPath: picturesDir];
                    
    for ( i = 0; i < [picArray count]; i++ ) {
        NSString	*pPath = [NSString stringWithFormat: @"%@/%@",
                                    picturesDir, [picArray objectAtIndex: i]];
                                    
        if ( [manager fileExistsAtPath: pPath isDirectory: &isDir]
                && ! isDir ) {
            NSLog( @"%@", pPath );
            [pics addObject: pPath];
        }
    }
    if ( [pics count] <= 0 ) {
        NSLog( @"no files in %@. exiting.", picturesDir );
        exit( 2 );
    }
    
    curimg = 0;
    [mainImage setImage: nil];
    bg = [[NSImage alloc] initWithContentsOfFile: [pics objectAtIndex: 0]];
    while ( bg == nil ) {
        [pics removeObjectAtIndex: 0];
        bg = [[NSImage alloc] initWithContentsOfFile: [pics objectAtIndex: 0]];
    }
    [mainImage setImage: bg];
    [NSApp activateIgnoringOtherApps: YES];
    timer = [NSTimer scheduledTimerWithTimeInterval: SLIDETIME
                        target: self
                        selector: @selector( changeBackground )
                        userInfo: nil
                        repeats: YES];
    [bg release];
}

- ( void )changeBackground
{
    NSImage	*bg;
    
    if ( curimg >= [pics count] ) curimg = 0;
    if ( [pics count] <= 0 ) {
        NSLog( @"No valid images!" );
        exit( 2 );
    }
    
    //NSLog( @"setting image to %@", [pics objectAtIndex: curimg] );
    
    bg = [[NSImage alloc] initWithContentsOfFile: [pics objectAtIndex: curimg]];
    while ( bg == nil ) {
        [pics removeObjectAtIndex: 0];
        bg = [[NSImage alloc] initWithContentsOfFile: [pics objectAtIndex: 0]];
    }
    
    [mainImage setImage: nil];
    [mainImage setImage: bg];
    [bg release];
    curimg++;
}

- ( void )dealloc
{
    [pics release];
    [super dealloc];
}

@end
